import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

import java.net.*;
import java.io.*;
import java.util.*;

import java.sql.*;

import zyh.util.ThreadPool;//A thread pool class

import zyh.robot.*;
import zyh.robot.TextHTML.TextHTMLFilter;//A parser for text and HTML documents

/** An index robot  demo*/

public class IndexRobot extends java.applet.Applet implements MouseListener
{
	boolean isApplication=false;

	TextField jdbcDriverNameTextField =null;
	TextField jdbcurlTextField =null;
	TextField httpProxyTextField =null;
	TextField indexURLTextField =null;
	Button indexButton=null;
	Button quitButton=null;
	TextArea echoTextArea =null;
	
	private static final boolean debug=true;

	private String jdbcDriverName=null;

	private Index index=null;
 
	public void init()
	{
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.weightx=1.0;
		constraints.weighty=0.0;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.NONE;
		constraints.gridwidth = GridBagConstraints.REMAINDER;

		GridBagLayout layout = new GridBagLayout();		

		setLayout(layout);
		setFont(new Font("Ariel", Font.PLAIN, 14));
		setBackground(Color.gray);

		Label label = new Label("Enter your JDBC 2.0 driver name: e.g., zyh.sql.dbf.DBFDriver");
		label.setFont(new Font("Ariel", Font.ROMAN_BASELINE,14));
//		label.setFont(new Font("Ariel", Font.PLAIN,14));
		label.setForeground(Color.white);
		constraints.anchor = GridBagConstraints.WEST;
		layout.setConstraints(label, constraints);
		add(label);

		jdbcDriverNameTextField = new TextField(40);
		jdbcDriverNameTextField.setText("zyh.sql.dbf.DBFDriver");
		jdbcDriverNameTextField.setEditable(true);
		jdbcDriverNameTextField.setBackground(Color.white);
		constraints.anchor = GridBagConstraints.CENTER;
		layout.setConstraints(jdbcDriverNameTextField, constraints);
		add(jdbcDriverNameTextField);

		label = new Label("Enter your JDBC url: e.g., jdbc:DBF:/.");
		label.setFont(new Font("Ariel", Font.PLAIN,14));
		label.setForeground(Color.white);
		constraints.anchor = GridBagConstraints.WEST;
		layout.setConstraints(label, constraints);
		add(label);

		jdbcurlTextField = new TextField(40);
		jdbcurlTextField.setText("jdbc:DBF:/../output");
		jdbcurlTextField.setEditable(true);
		jdbcurlTextField.setBackground(Color.white);
		constraints.anchor = GridBagConstraints.CENTER;
		layout.setConstraints(jdbcurlTextField, constraints);
		add(jdbcurlTextField);

		String host=isApplication?"localhost":getCodeBase().getHost();
		if(host.length()==0)host="localhost";

		label = new Label("Enter your http proxy if you are using: e.g., "+host+":8080");
		label.setFont(new Font("Ariel", Font.PLAIN,14));
		label.setForeground(Color.white);
		constraints.anchor = GridBagConstraints.WEST;
		layout.setConstraints(label, constraints);
		add(label);

		httpProxyTextField = new TextField(40);
		httpProxyTextField.setText("");
		httpProxyTextField.setEditable(true);
		httpProxyTextField.setBackground(Color.white);
		constraints.anchor = GridBagConstraints.CENTER;
		layout.setConstraints(httpProxyTextField, constraints);
		add(httpProxyTextField);

		label = new Label("Enter an index URL(e.g., http://www.mokabyte.it)");
		label.setFont(new Font("Ariel", Font.PLAIN, 14));
		label.setForeground(Color.white);
		constraints.anchor = GridBagConstraints.WEST;
		layout.setConstraints(label, constraints);
		add(label);

		indexURLTextField = new TextField(25);
		indexURLTextField.setText("http://"+host);
		indexURLTextField.setEditable(true);
		indexURLTextField.setBackground(Color.white);
		constraints.anchor = GridBagConstraints.CENTER;
		layout.setConstraints(indexURLTextField, constraints);
		add(indexURLTextField);

		indexButton = new Button(" Index ");
		indexButton.addMouseListener(this);
		constraints.fill = GridBagConstraints.BOTH;
		layout.setConstraints(indexButton, constraints);
		add(indexButton);

		quitButton = new Button(isApplication?" Quit ":" Stop ");
		quitButton.addMouseListener(this);
		constraints.fill = GridBagConstraints.BOTH;
		layout.setConstraints(quitButton, constraints);
		add(quitButton);

		label = new Label("Index reports:");
		label.setFont(new Font("Ariel", Font.PLAIN,14));
		label.setForeground(Color.blue);
		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.NONE;
		layout.setConstraints(label, constraints);
		add(label);
		
		constraints.weighty=1.0;
		echoTextArea = new TextArea(20,120);
		echoTextArea.setEditable(false);
		layout.setConstraints(echoTextArea, constraints);
		echoTextArea.setForeground(Color.white);
		echoTextArea.setBackground(Color.black);
		echoTextArea.setText("");
		add(echoTextArea);
		
		setVisible(true);
	}
	
	public void mouseClicked(MouseEvent me)
	{
		try
		{
			Object target =me.getSource();
			if (target == indexButton)
			{
				if(indexButton.getLabel().equals(" Stop "))
				{
					if(index!=null)
					{
						indexButton.setEnabled(false);
						index.stop();
						index=null;
					}
					else
						indexButton.setLabel(" Index ");
				}
				else
				{
					indexButton.setLabel(" Stop ");
					indexButton.setEnabled(true);
					if(index!=null)index.stop();
					index=new Index();
				}
			}
			else if (target == quitButton)
			{
				if(index!=null)index.stop();
				if(isApplication)System.exit(0);
			}
		}
		catch( Exception e)
		{
			indexButton.setEnabled(true);
			e.printStackTrace();
			System.out.println(e.getMessage());
			echoTextArea.setText(e.getMessage());
		}
	}

	/**
	 * Create index dababase if it is not existent.
	 *
	 * It need a JDBC 2.0 driver which supports the scrollable
	 * updatable resultset. Some serializable objects will be 
	 * held in some BLOB, OBJECT or VARBINARY columns. 
	 *
	 *
	 * If your JDBC driver can not utilize ResultSet.getObject(int columnIndex)
	 * and ResultSet.updateObject(int columnIndex, Object x)
	 * to manage Serializable objects. You can also use a 
	 * VARCHAR or VARBINARY column to cope with Serializable objects.
	 * The relative code like:
	 *
	 * <pre>
			java.io.InputStream is= rs.getBinaryStream(columnIndex);
			if(is==null)
				obj=null;
				else
				{
					java.io.ObjectInputStream ois=new java.io.ObjectInputStream(is);
					obj=ois.readObject(); 
				}

			java.io.ByteArrayOutputStream baos=new java.io.ByteArrayOutputStream(1024);
			java.io.ObjectOutputStream  oos = new java.io.ObjectOutputStream(baos);
			oos.writeObject( Your Serializable object );
			rs.updateBytes( columnIndex, baos.toByteArray());
	 * </pre>
	 *
	 * Only zyh.robot.URLManager, zyh.robot.ContentSorter,
	 * IndexRobot and Search have ever operated the index dabase through JDBC interface.
	 */
	protected void initIndexDatabase(String jdbcurl,
		String urlsTableName,
		String keywordTableName,
		String stopwordTableName,
		PrintWriter log
		)throws Exception
	{
		Connection con = DriverManager.getConnection(jdbcurl, "", "");
		Statement stmt = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,ResultSet.CONCUR_UPDATABLE);
		ResultSet rs;
		try
		{
			rs=stmt.executeQuery("select * from "+stopwordTableName+" where false");
			log.println("Found the existed index database.");
			log.flush();
			return;//The stopword table has existed.
		}
		catch(Exception se)//The stopword table has not existed.
		{}

		log.println("Try to load the stopword list from \"stopwords.txt\".");
		log.flush();
		//Read the plain text which contains the stopword list
		BufferedReader in;
		if(isApplication)
			in= new BufferedReader(new FileReader("stopwords.txt"));
		else
			in= new BufferedReader(new FileReader(new java.net.URL(this.getDocumentBase(),"stopwords.txt").getFile()));

		//Create the stopword table
		stmt.executeUpdate("create table "+stopwordTableName+"(stopword char(16))");
		rs=stmt.executeQuery("select stopword from "+stopwordTableName);
		rs.moveToInsertRow();

		log.println("Succeeded in create the stopword table.");
		log.flush();

		int ncount=0;
		String word;
		while((word=in.readLine())!=null)
		{
			word=word.trim();
			if(word.length()==0)continue;
			rs.updateString(1, word);
			rs.insertRow();
			ncount++;
		}
		log.println("Inserted "+ncount+" stopwords into the stopword table.");
		log.flush();
		in.close();
		rs.close();

		String objectDatatype;//The name of JAVA_OBJECT data type for a specific JDBC driver.
		if(jdbcurl.startsWith("jdbc:DBF:"))//DBF driver
			objectDatatype="JAVA_OBJECT";
		else
			objectDatatype="OBJECT";//Use a OBJECT data type for byte[].
//			objectDatatype="BLOB";//Use a BLOB data type for byte[].
//			objectDatatype="VARBINARY";//Use a VARBINARY data type for byte[].

		log.println("Choose "+objectDatatype+" to hold the serializable object.");
		log.flush();


		//Create the url table
		stmt.executeUpdate("create table "+urlsTableName+"(urlid int, url char(100),urlstatus int, urlmessage char(20), urlsummary "+objectDatatype+","
			+"urltime TIMESTAMP, keywords "+objectDatatype
			+")");
		log.println("Succeeded in create the url table.");
		log.flush();

		//Create the keyword table
		stmt.executeUpdate("create table "+keywordTableName+"(wordid int, keyword char(20), urls "+objectDatatype+")");
		log.println("Succeeded in create the keyword table.");
		log.flush();
	}

	public void mousePressed(MouseEvent me)
	{
	}
	public void mouseReleased(MouseEvent me)
	{
	}
	public void mouseEntered(MouseEvent me)
	{
	}
	public void mouseExited(MouseEvent me)
	{
	}

	public static void main(String args[]) throws Exception
	{
		try
		{
			// Create the frame this applet will run in
			Frame appletFrame = new Frame("An index robot demo");
			appletFrame.setLayout(new GridLayout(1,0));
			
			IndexRobot indexRobot=new IndexRobot();
			indexRobot.isApplication=true;

			// Add the IndexRobot applet to the frame
			appletFrame.add(indexRobot);
			
			indexRobot.init();
			indexRobot.start();

			appletFrame.setSize(600, 420);
			appletFrame.show();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.out.println(e);
			System.exit(1);
		}
	}

	/*
	 * Run the index routine in an independent thread so that it can be stopped anytime
	 */
	private class Index implements Runnable
	{
		private ThreadPool threadPool;
		public Index()
		{
			threadPool=new ThreadPool(5,10);
			threadPool.run(this);
		}

		private Thread myThread=null;
		/*
		 * Stop the index routine
		 */
		public void stop()
		{myThread=null;}

		private int interval=10000;
		public void run()
		{
			echoTextArea.setText("");
//			PrintWriter out=new PrintWriter(System.out);
			PrintWriter out=new PrintWriter(new TextAreaOut(echoTextArea));
			try
			{

				//Set JDBC driver
				String text=jdbcDriverNameTextField.getText().trim();
				if(!text.equals(jdbcDriverName))
				{
					jdbcDriverName=text;
					Class.forName(jdbcDriverName).newInstance();
					out.println("Loaded jdbc driver:"+jdbcDriverName);
					out.flush();
				}

				//Set JDBC url
				String jdbcurl=jdbcurlTextField.getText().trim();
				out.println("Set jdbc url:"+jdbcurl);
				out.flush();

				//Set Http proxy parameters
				java.net.InetAddress proxyInetAddress=null;
				int proxyPort=8080;

				text=httpProxyTextField.getText().trim();
				if(text.length()>0)
				{
					int index=text.indexOf(":");
					String host;
					if(index<0)
						host=text;
					else
					{
						host=text.substring(0,index);
						proxyPort=Integer.parseInt(text.substring(index+1));
					}
					proxyInetAddress=InetAddress.getByName(host);
					out.println("Set http proxy to "+host+":"+proxyPort);
					out.flush();
				}

				//Set the start url
				URL baseurl=new URL(indexURLTextField.getText().trim());
				out.println("Set the start url:"+baseurl);
				out.flush();

				String[] interestedIdentifiers=new String[1];
				interestedIdentifiers[0]=baseurl.getHost();//Focuses on the URL belongs this host

				if(interestedIdentifiers!=null
					&& interestedIdentifiers.length>0)
				{			
					out.println("The interested identifier"+
						(interestedIdentifiers.length>1?"s are ":" is ")
						+interestedIdentifiers[0]);
					if(interestedIdentifiers.length>1)
					{
						for(int i=1;i<interestedIdentifiers.length-1;i++)
							out.println(interestedIdentifiers[i]);
						out.println(" and "+interestedIdentifiers[interestedIdentifiers.length-1]);
					}
				}
				out.flush();

				String urlsTableName="urls";
				String keywordTableName="keywords";
				String stopwordTableName="stopwords";

				initIndexDatabase(jdbcurl,urlsTableName,keywordTableName,stopwordTableName,out);//Initialize index dababase

				int maxDepth=3;
				out.println("The maximal recursive depth:"+maxDepth);

				//Please provide your email so that server maintainers can contact you in case of problems
				//if you use it to visit Internet sites.
				String mailbox="zhaoyh@bigfoot.com";


				URLManager urlManager=new URLManager(maxDepth,interestedIdentifiers,mailbox,proxyInetAddress,proxyPort,jdbcurl,urlsTableName);

				if(debug)
				{
					urlManager.setLogWriter(out);
					ContentManager.setLogWriter(out);
				}

				urlManager.addURL(baseurl);

				ContentFilter textHTMLFilter=new TextHTMLFilter();
				ContentManager.registerContentFilter(textHTMLFilter);
				//Rigister TextHTMLFilter to cope with the content types of "text/css" and "text/xml"
				ContentManager.registerContentFilter("text/css",textHTMLFilter);
				ContentManager.registerContentFilter("text/xml",textHTMLFilter);
				//Rigister TextHTMLFilter to cope with all subtypes of "text" type
	//			ContentManager.registerContentFilter("text",textHTMLFilter);

				ContentProcessor contentSorter=new ContentSorter(jdbcurl,keywordTableName,stopwordTableName);

				/*
				 * requestInterval Requests 1 document per n milliseconds
				 * Suggestion: Using 60000~300000 milliseconds to avoid the potential overload of the destination host
				 * Warning: Using a smaller value for requestInterval only when you're exploring your own website
				 */
				long requestInterval=0;//60000
				int robotNumber=5;
				Robot[] robots=new Robot[robotNumber];
				for(int i=0;i<robots.length;i++)
				{
					robots[i]=new Robot(requestInterval,urlManager,threadPool,proxyInetAddress,proxyPort);
					ContentManager.registerContentProcessor(robots[i],contentSorter);
				}

//				if(!isApplication)
					out=new PrintWriter(new FileOutputStream("URLReport.html"));

				myThread=Thread.currentThread();
				while(myThread!=null)
				{
					try
					{
						Thread.sleep(interval);
					}
					catch(InterruptedException ie){}


					int i;
					for(i=0;i<robots.length;i++)
					{
						if(!robots[i].isIdle())
							break;
					}
					if(i>=robots.length)break;
				}
				for(int i=0;i<robots.length;i++)
					robots[i].stop();
				threadPool.close();

				contentSorter.close();
				urlManager.report(out);
				if(!isApplication)
				{
					URL u = new URL(getCodeBase().toString() + "URLReport.html");
					getAppletContext().showDocument(u);
				}
			}
			catch(Exception e)
			{
				out.println(e+"");
				out.flush();
			}
			indexButton.setLabel(" Index ");
			indexButton.setEnabled(true);
		}
	}
	/*
	 * simulate an output stream by a TextArea object
	 */
	private class TextAreaOut extends OutputStream
	{
		TextArea ta;
		TextAreaOut(TextArea ta)
		{
			this.ta=ta;
		}
		private ArrayList rows=new ArrayList(200);//Used to record all rows in this text area.
		public void close()
		{
			ta.setText("");
			rows.clear();
		}		
        public void flush()
		{
		}
		public void write(int b)
		{
			ta.append(((byte)b)+"");
		}
		public void write(byte[] b)
		{
			if(b!=null)
				ta.append(new String(b));
		}
		public void write(byte[] b, int off, int len)
		{
			if(b!=null)
			{
				String row=new String(b,off,len);
				if(rows.size()>=200)
				{
					for(int i=rows.size()-1;i>50;i--)
						rows.remove(i);
					ta.setText("");
					for(int i=0;i<rows.size();i++)
						ta.append((String)rows.get(i));
					ta.append("Cleared many echo messages.\r\n");
				}
				rows.add(row);
				ta.append(row);
			}
		}
	} 
}