import java.net.*;
import java.io.*;

/** A simple URLs extractor for demo purpose
 *
 * All the Java source code files in SimpleExample are generated by JLex and
 * CUP according to the corresponding specification of <a href="SimpleExample/doc-files/Yylex">Yylex</a>
 * and <a href="SimpleExample/doc-files/URLExtractor.cup">URLExtractor.cup</a>. 
 *
 * <a href="http://www.cs.princeton.edu/~appel/modern/java/JLex/">JLex</a> is a lexical analyzer generator for Java,
 * and <a href="http://www.cs.princeton.edu/~appel/modern/java/CUP/">CUP</a> is correspondingly a LALR parser constructor.


 */

public class URLExtractor
{
	public static void main(String args[]) throws Exception
	{
		try
		{
			if(args.length!=1)
			{
				System.out.println("Usage: java URLExtractor httpURL");
				System.out.println("For example, java URLExtractor http://pii300/index.html");
				System.exit(1);
			}

			URL url = new URL(args[0]); //Create a java.net.URL object according to a specific URL in String form.
			if(url.getPort()==-1) //The port is not set
				url = new URL(url.getProtocol(),url.getHost(),80,url.getFile()); //Set the default port to 80
			if(!url.getProtocol().equals("http"))
			{
				System.out.println("The protocol should be http but you are using "+url.getProtocol()+" for "+url);
				System.exit(1);
			}

			Socket socket = new Socket(url.getHost(), url.getPort()); //Create a stream socket and connect it to the specified port number on the web host.
			socket.setSoTimeout(120000); //Throw a java.io.InterruptedIOException if the timeout expires.
			java.io.PrintStream out = new java.io.PrintStream(socket.getOutputStream()); //Get a print stream from the output stream for this socket.
			out.print("GET "+url.getFile()+" HTTP/1.0\r\n"); //Construct an HTTP GET request line with the relative URL and HTTP/1.0 version
			out.print("Referer: http://pii300/java.html\r\n"); //Tell the server the referenced URL from which the request URL was obtained so that the obsolete or mistyped link can be traced for maintenance.
			out.print("User-Agent: Java Robot/1.01\r\n"); //Tell the server the robot name so that the server can distinguish it from browsers.
			out.print("From: robotmaster@companyname.com\r\n"); //Provide your mailbox so that server maintainer can contact you in case of problems.
			out.print("Pragma: no-cache\r\n"); //Ignore the caches.
			out.print("Host: "+url.getHost()+":"+url.getPort()+"\r\n"); //Provide the Internet host and port number of the relative URL being requested.
			out.print("Accept: */*\r\n"); //Accept all media types and their subtypes.
			out.print("Cookie: "+"sessionid=126\r\n"); //Submit a Cookie if the web site allots it a Cookie.
			out.print("\r\n"); //A blank line indicates the end of the header fields.
			out.flush(); //Flush the stream.
			InputStream in=socket.getInputStream(); //Get an input stream for this socket.

			java.util.ArrayList urls=SimpleExample.parser.getURLs(in);//Extract URLs from this webpage.
			for(int i=0;i<urls.size();i++)
			{
				try
				{
					System.out.println(i+": "+(new URL(url,(String)urls.get(i))));
				}
				catch(Exception e)
				{
					System.out.println(i+": "+urls.get(i));
				}
			}

			socket.close(); //Closes this socket.
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.out.println(e);
			System.exit(1);
		}
	}
}