package zyh.robot;

/**
 * The interface that every content processor class must implement.
 * ContentProcessor decides on the index policy
 * and does actual index process.
 *
 * 
 *
 * <P>The zyh.robot framework allows for multiple content processors.
 *
 * @see Content 
 */
public interface ContentProcessor
{
	/**
	 * Sort and Index a content object.
	 *
	 * @return a wordID array of all keywords from the content
	 *
	 * @param content an Content object
	 * @exception SQLException if an index database access error occurs
	 */
	public int[] getWordIDs(Content content)throws java.sql.SQLException;

	/**
	 * Uses to release all occupied resource by this content processor
	 *
	 * @exception SQLException if an index database access error occurs
	 */
	public void close()throws java.sql.SQLException;
}