package zyh.robot.TextHTML;

import zyh.robot.ContentFilter;
import zyh.robot.Content;

/**
 * The TextHTMLFilter clase is used to filter the content 
 * of Text or HTML documents.
 *
 * All the Java source code files in zyh.robot.TextHTML,
 * except for TextHTMLFilter.java, are generated by JLex and
 * CUP according to the corresponding specification of <a href="doc-files/Yylex">Yylex</a>
 * and <a href="doc-files/TextHTML.cup">TextHTML.cup</a>. 
 * <a href="http://www.cs.princeton.edu/~appel/modern/java/JLex/">JLex</a> is a lexical analyzer generator for Java,
 * and <a href="http://www.cs.princeton.edu/~appel/modern/java/CUP/">CUP</a> is correspondingly a LALR parser constructor.
 *
 *  For full-text indexing purpose, your HTML parser need only focus
 *  on some important embedded tags, not be fully compliant with the
 *  latest HTML4.01 specification. Your HTML filter should recognize
 *  seven HTML elements or attributes thereinafter:<br>
 *  1. URL of embedded image, frame or script: For example, &lt;IMG src= "http://www.somecompany.com/People/Ian/vacation/family.png" alt="A photo of my family at the lake.">, &lt;FRAME src="contents_of_frame1.html">, &lt;IFRAME src="foo.html" width="400" height="500" scrolling="auto" frameborder="1">, and &lt;SCRIPT type="text/vbscript" src= "http://someplace.com/progs/vbcalc"> &lt;/SCRIPT>.<br>
 *  2. Background image(Deprecated): For example, &lt;BODY background ="bk1.gif" bgcolor="white" text="black" link="red" alink="fuchsia" vlink="maroon">... document body...&lt;/BODY>.<br>
 *  3. URL for linked resource or base URL: For example, &lt;A href= "http://www.w3.org/" charset="ISO-8859-1">W3C Web site&lt;/A>, &lt;A href= "./one.html#anchor-one"> anchor one&lt;/A>, &lt;a href= "mailto:someone@somecompany.com">some comments&lt;/A>, &lt;BASE href= "http://www.aviary.com/products/intro.html">, and &lt;LINK media="print" title="The manual in postscript" type="application/postscript" rel="alternate" href= "http://someplace.com/manual/postscript.ps">.<br>
 *  4. Document title: For example, &lt;TITLE>A study of population dynamics&lt;/TITLE>.<br>
 *  5. Author, keywords and description in generic meta information: For example, &lt;META name="author" content="John Doe">, &lt;META name="keywords" lang="en-us" content="vacation, Greece, sunshine">, and &lt;META name="description" content="Idyllic European vacations">.'<br>
 *  6. When to fetch a fresh copy of the associated document in generic meta information: For example, &lt;META http-equiv="Expires" content="Tue, 20 Aug 1996 14:25:27 GMT"><br>
 *  7. Headings from H1 to H6: For example, &lt;H1>Forest elephants&lt;/H1>.<br>
 *
 *  You can also extract URLs from the HTML elements of APPLET, 
 *  OBJECT, BLOCKQUOTE, INS, and DEL by appending simply the 
 *  corresponding symbols and grammar in the definition files 
 *  of Yylex and TextHTML.cup. Words, including JavaScript code,
 *  within the remaining HTML tags, can be plainly transacted into
 *  the text part of content objects since content processor will 
 *  purify all content in content objects at the next phase.
 *
 *  Sometimes it's impossible to extract all URLs from web page
 *  because some URLs are possibly loaded dynamically by JavaScript.
 *
 */
public class TextHTMLFilter implements ContentFilter
{
	private static final String[] acceptedContenttypes={"text/plain","text/html"};

	/**
	 * Return an String array which contains all supported content types
	 */
	public String[] getAcceptedContenttype()
	{
		return acceptedContenttypes;
	}

	/**
	 * Attempts to get a content from a given input stream.
	 *
	 * @return a Content from the input stream
	 *
	 * @param InputStream an input stream
	 * @exception IOException if an input stream access error occurs
	 */
	public Content getContent(java.io.InputStream in)throws java.io.IOException
	{
		return zyh.robot.TextHTML.parser.getContent(in);
	}
}